#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
BW2QSD: Bridging Brightway2 and QSD packages for LCA

This module is copied from the EcoInventDownLoader (eidl) repository at
https://github.com/haasad/EcoInventDownLoader
with only minor bug fix and modification to be compatible with BW2QSD.

The reason for including this module in BW2QSD rather than adding EcoInventDownLoader
as a dependent package is because EcoInventDownLoader does not have a PyPI release,
and installing it via conda requires the installation of the entire Brightway2 framework
via conda, which has some unfixed bugs, and the use of both pip and conda in installation
could be overwhelming for users.

This module is under dual licenses:

    EcoInventDownLoader (MIT)
    Please refer to https://github.com/haasad/EcoInventDownLoader/blob/master/LICENSE
    for license details.

    BW2QSD (University of Illinois/NCSA)
    Please refer to https://github.com/QSD-Group/BW2QSD/blob/main/LICENSE.txt
'''

import os
import glob

import appdirs


class EidlStorage():
    def __init__(self):
        self.eidl_dir = appdirs.user_data_dir(
            appname='EcoInventDownLoader',
            appauthor='EcoInventDownLoader'
        )

        if not os.path.isdir(self.eidl_dir):
            os.makedirs(self.eidl_dir)

    @property
    def stored_dbs(self):
        paths = glob.glob(os.path.join(self.eidl_dir, '*.7z'))
        db_dict = {os.path.split(p)[1]: p for p in paths}
        return db_dict

    def clear_stored_dbs(self):
        for db in self.stored_dbs.values():
            os.remove(db)


eidlstorage = EidlStorage()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
BW2QSD: Bridging Brightway2 and QSD packages for LCA

This module is copied from the EcoInventDownLoader (eidl) repository at
https://github.com/haasad/EcoInventDownLoader
with only minor bug fix and modification to be compatible with BW2QSD.

The reason for including this module in BW2QSD rather than adding EcoInventDownLoader
as a dependent package is because EcoInventDownLoader does not have a PyPI release,
and installing it via conda requires the installation of the entire Brightway2 framework
via conda, which has some unfixed bugs, and the use of both pip and conda in installation
could be overwhelming for users.

This module is under dual licenses:

    EcoInventDownLoader (MIT)
    Please refer to https://github.com/haasad/EcoInventDownLoader/blob/master/LICENSE
    for license details.

    BW2QSD (University of Illinois/NCSA)
    Please refer to https://github.com/QSD-Group/BW2QSD/blob/main/LICENSE.txt
'''

import os
import string
import tempfile
import getpass
import subprocess

import requests
from bs4 import BeautifulSoup
import brightway2 as bw

from .storage import eidlstorage


class EcoinventDownloader:
    def __init__(self, username=None, password=None, version=None,
                 system_model=None, outdir=None, **kwargs):
        self.username = username
        self.password = password
        self.version = version
        self.system_model = system_model
        self.outdir = outdir

    def run(self):
        if self.check_stored():
            return
        if self.username is None or self.password is None:
            self.username, self.password = self.get_credentials()
        print('logging in to ecoinvent homepage...')
        self.login()
        self.db_dict = self.get_available_files()
        print('login successful!')
        if (self.version, self.system_model) not in self.db_dict.keys():
            self.version, self.system_model = self.choose_db()
        if self.check_stored():
            return

        print('downloading {} {} ...'.format(self.system_model, self.version))
        self.download()
        print('download finished!: {}\n'.format(self.out_path))

    @property
    def file_name(self):
        fn = '{}{}.7z'.format(self.system_model, str(self.version).replace('.', ''))
        return fn

    def check_stored(self):
        if self.file_name in eidlstorage.stored_dbs:
            self.out_path = eidlstorage.stored_dbs[self.file_name]
            print('database already downloaded')
            return True
        else:
            return False

    def get_credentials(self):
        un = input('ecoinvent username: ')
        pw = getpass.getpass('ecoinvent password: ')
        return un, pw

    def login(self):
        self.session = requests.Session()
        logon_url = 'https://v33.ecoquery.ecoinvent.org/Account/LogOn'
        post_data = {'UserName': self.username,
                     'Password': self.password,
                     'IsEncrypted': 'false',
                     'ReturnUrl': '/'}
        try:
            self.session.post(logon_url, post_data, timeout=20)
        except (requests.ConnectTimeout, requests.ReadTimeout, requests.ConnectionError) as e:
            self.handle_connection_timeout()
            raise e

        success = bool(self.session.cookies)
        self.login_success(success)

    def login_success(self, success):
        if not success:
            print('Login failed')
            self.username, self.password = self.get_credentials()
            self.login()

    def handle_connection_timeout(self):
        print('The request timed out, please check your internet connection!')
        if eidlstorage.stored_dbs:
            print(
                'You have the following databases stored:\n\t{}\n'.format(
                    '\n\t'.join(eidlstorage.stored_dbs.keys())) +
                'You can use these offline by adding the corresponding `version` and `system_model` keywords\n' +
                "Example: eidl.get_ecoinvent(version='3.5', system_model='cutoff')"
            )

    def get_available_files(self):
        files_url = 'https://v33.ecoquery.ecoinvent.org/File/Files'
        try:
            files_res = self.session.get(files_url, timeout=20)
        except (requests.ConnectTimeout, requests.ReadTimeout, requests.ConnectionError) as e:
            self.handle_connection_timeout()
            raise e
        soup = BeautifulSoup(files_res.text, 'html.parser')
        file_list = [l for l in soup.find_all('a', href=True) if
                     l['href'].startswith('/File/File?')]
        link_dict = {f.contents[0]: f['href'] for f in file_list}
        link_dict = {
            k.replace('-', ''):v for k, v in link_dict.items() if k.startswith('ecoinvent ') and
            k.endswith('ecoSpold02.7z') and not 'lc' in k.lower()
        }
        db_dict = {
            tuple(k.replace('ecoinvent ', '').split('_')[:2:]): v for k, v in link_dict.items()
        }
        return db_dict

    def choose_db(self):
        versions = {k[0] for k in self.db_dict.keys()}
        version_dict = dict(zip(string.ascii_lowercase,
                                sorted(versions, reverse=True)))
        print('\n', 'available versions:')
        for k, version in version_dict.items():
            print(k, version)
        while True:
            version = input('version: ')
            if version in versions or version in version_dict.keys():
                break
            else:
                print('Enter version number or letter')
        system_models = {k[1] for k in self.db_dict.keys() if k[0] == version_dict.get(version, version)}
        sm_dict = dict(zip(string.ascii_lowercase, sorted(system_models)))
        print('\n', 'system models:')
        for k, sm in sm_dict.items():
            print(k, sm)
        while True:
            sm = input('system model: ')
            if sm in system_models or sm in sm_dict.keys():
                break
            else:
                print('Enter system model or letter')
        dbkey = (version_dict.get(version, version),
                 sm_dict.get(sm, sm))
        return dbkey

    def download(self):
        url = 'https://v33.ecoquery.ecoinvent.org'
        db_key = (self.version, self.system_model)
        try:
            file_content = self.session.get(url + self.db_dict[db_key], timeout=60).content
        except (requests.ConnectTimeout, requests.ReadTimeout, requests.ConnectionError) as e:
            self.handle_connection_timeout()
            raise e

        if self.outdir:
            self.out_path = os.path.join(self.outdir, self.file_name)
        else:
            self.out_path = os.path.join(os.path.abspath('.'), self.file_name)

        with open(self.out_path, 'wb') as out_file:
            out_file.write(file_content)

    def extract(self, target_dir):
        extract_cmd = ['py7zr', 'x', self.out_path, target_dir]
        try:
            self.extraction_process = subprocess.Popen(extract_cmd)
            self.extraction_process.wait()
        except FileNotFoundError as e:
            if "PYCHARM_HOSTED" in os.environ:
                print('It appears the EcoInventDownLoader is run from PyCharm. ' +
                      'Please make sure you select the the correct conda environment ' +
                      'as your project interperter or run your script/command in a ' +
                      'Python console outside of PyCharm.')
            raise e


def get_ecoinvent(db_name=None, auto_write=False, download_path=None, store_download=True, **kwargs):

    """
    Download and import ecoinvent to current brightway2 project
    Optional kwargs:
        db_name: name to give imported database (string) default is downloaded filename
        auto_write: automatically write database if no unlinked processes (boolean) default is False (i.e. prompt yes or no)
        download_path: path to download .7z file to (string) default is download to temporary directory (.7z file is deleted after import)
        store_download: store the .7z file for later reuse, default is True, only takes effect if no download_path is provided
        username: ecoinvent username (string)
        password: ecoivnent password (string)
        version: ecoinvent version (string), eg '3.5'
        system_model: ecoinvent system model (string), one of {'cutoff', 'apos', 'consequential'}
    """
    with tempfile.TemporaryDirectory() as td:
        if download_path is None:
            if store_download:
                download_path = eidlstorage.eidl_dir
            else:
                download_path = td

        downloader = EcoinventDownloader(outdir=download_path, **kwargs)
        downloader.run()
        downloader.extract(target_dir=td)

        if not db_name:
            db_name = downloader.file_name.replace('.7z', '')
        datasets_path = os.path.join(td, 'datasets')
        importer = bw.SingleOutputEcospold2Importer(datasets_path, db_name)

    if 'biosphere3' not in bw.databases:
        if auto_write:
            bw.bw2setup()
        else:
            print('No biosphere database present in your current ' +
                  'project: {}'.format(bw.projects.current))
            print('You can run "bw2setup()" if this is a new project. Run it now?')
            if input('[y]/n ') in {'y', ''}:
                bw.bw2setup()
            else:
                return

    importer.apply_strategies()
    datasets, exchanges, unlinked = importer.statistics()

    if auto_write and not unlinked:
        print('\nWriting database {} in project {}'.format(
            db_name, bw.projects.current))
        importer.write_database()
    else:
        print('\nWrite database {} in project {}?'.format(
            db_name, bw.projects.current))
        if input('[y]/n ') in {'y', ''}:
            importer.write_database()


def get_ecoinvent_cli():
    downloader = EcoinventDownloader()
    downloader.run()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
BW2QSD: Bridging Brightway2 and QSD packages for LCA

This module is developed by:
    Yalin Li <mailto.yalin.li@gmail.com>

This module is under the University of Illinois/NCSA Open Source License.
Please refer to https://github.com/QSD-Group/BW2QSD/blob/main/LICENSE.txt
for license details.
'''

'''
TODO:
    Improve search ability, update docs

'''


import sys, os
import pandas as pd
import brightway2 as bw2
from collections.abc import Iterable
from warnings import warn
from bw2data.backends.peewee import Activity
from .utils import export_df, format_name

__all__ = ('CFgetter',)


def _filter_inds(inds, cats, include):
    for n, cat in enumerate(cats):
        if cat:
            if isinstance(cat, str):
                cat = (cat, )
            elif not isinstance(cat, Iterable):
                raise TypeError(f'"{cat}" can only be str or iterable, ' \
                                f'not {type(cat).__name__}.')
            if include is True:
                for string in cat:
                    string_lower = string.lower()
                    inds = [ind for ind in inds if string_lower in ind[n].lower()]
            else:
                for string in cat:
                    string_lower = string.lower()
                    inds = [ind for ind in inds if not string_lower in ind[n].lower()]

    return inds


class CFgetter:
    '''
    To get environmental impact characterization factors from databases through
    `Brightway2`, users can choose different life cycle inventory (LCI) databases
    and life cycle impact assessment (LCIA).

    Tip
    ---
    Basic steps:

        [1] Load database (you can only load one at a time).

        [2] Load indicators (you can load multiple).

        [3] Load activities (you can load multiple).


    .. note::

        You need to first download the database using :class:`DataDownloader`.


    See Also
    --------
    `Brightway2 <2.docs.brightway.dev>`_

    '''

    __slots__ = ('name', '_database', '_indicators', '_activities', '_CFs')

    def __init__(self, name):
        self.name = name
        self._database = None
        self._indicators = set()
        self._activities = {}
        self._CFs = None

    def __repr__(self):
        return f'<CFgetter: {self.name}>'

    def load_database(self, database):
        '''
        Load designated database.

        Parameters
        ----------
        database: str
            Name of the database.
        '''
        db_lower = database.lower()

        if db_lower not in bw2.databases:
            raise ValueError(f'Database "{database}" not available. ' \
                             'Please first download the data using `DataDownloader`.')
        else:
            self._database = db = bw2.Database(database)

        print(f'Database {db} with {len(db)} inventories has been loaded.')


    def load_indicators(self, add=False, method='', method_exclude='',
                        category='', category_exclude='',
                        indicator='', indicator_exclude=''):
        '''
        Select and/or load designated impact indicators.

        Parameters
        ----------
        add : bool
            Whether to include the indicators in impact assessment for characterization factors.
            If False, a dict of the indicators that satisfy the search criteria
            will be returned.
        method: str or Iterable
            Impact assessment method of the indicator (e.g., TRACI).
        method_exclude: str or Iterable
            Strings to be excluded from the method field.
        category: str or Iterable
            Category of the indicator (e.g., environmental impact).
        category_exclude: str or Iterable
            Strings to be excluded from the category field.
        indicator: str or Iterable
            Name of the indicator (e.g., global warming).
        indicator_exclude: str or Iterable
            Strings to be excluded from the indicator field.

        Tip
        ---
        Leave all fields as blank (i.e., '') if want all of the options.
        E.g., load_indicators() will return
        all indicators available in the package (800+ in total).

        '''
        indicators = bw2.methods.list
        indicators = _filter_inds(indicators, (method, category, indicator), True)
        indicators = _filter_inds(indicators,
                                  (method_exclude, category_exclude, indicator_exclude),
                                  False)

        if add:
            self._indicators = self._indicators.union(set(indicators))
            msg = 'indicator' if len(indicators) > 1 else 'indicators'
            print(f'\n{len(indicators)} {msg} loaded/updated for {self.name}.')

        else:
            return indicators


    def load_activities(self, string: str, add: bool, limit=20, show=False,
                        **kwargs):
        '''
        Select and/or load activities of interest.

        Parameters
        ----------
        string : str
            Search string.
        add : bool
            Whether to include the activities in impact assessment for characterization factors.
            If False, a dict of the activities that satisfy the search criteria
            will be returned.
        limit : int
            Maximum number of search results to return.
        show : bool
            Whether to print the detailed information associated with the activities.
        kwargs :
            Other keyword arguments that will be passed to `bw2data`.

        See Also
        --------
        :func:`search` in `bw2data SQLiteBackend <https://2.docs.brightway.dev/technical/bw2data.html#default-backend-databases-stored-in-a-sqlite-database>`_

        '''
        activities = self.database.search(string, limit=limit, **kwargs)
        act_dct = {act.as_dict()['name']: act for act in activities}

        for act in activities:
            if show:
                self.show_activity(act)

        if add:
            self._activities.update(act_dct)
            msg = 'activities' if len(act_dct) > 1 else 'activity'
            print(f'{len(act_dct)} {msg} loaded/updated for {self.name}.\n')

        else:
            return act_dct

    def show_activity(self, activity=None, **kwargs):
        '''
        Show detailed description about an activity.

        Parameters
        ----------
        activity : str or :class:`Activity`
            Name of the activity as str, or the :class:`Activity` in `bw2data`.
        kwargs :
            Other keyword arguments that will be passed to `bw2data`.

        .. note::

            When given as a str, the function will first try to search within
            the loaded activities, if there is no match,
            then it will search the entire database.

        '''
        if not activity:
            activities = [i for i in self.activities.values()]

        elif isinstance(activity, str):
            try:
                activities = [self.activities[activity]]
            except KeyError:
                activities = self.database.search(activity, **kwargs)
                if not activities:
                    raise ValueError(f'No search results for "{activity}".')

        else:
            if not isinstance(activity, Activity):
                raise TypeError(f'The input activity type "{type(activity).__name__}" is not valid.')
            else:
                activities = [activity]

        dfs = []
        for act in activities:
            act_dct = act.as_dict()
            name = act_dct['name']
            df = pd.DataFrame(
                data={name: [v for v in act_dct.values()]},
                index=[k for k in act_dct.keys()])
            print(f'{df.fillna("N/A")}\n')

            dfs.append(df)

        if len(dfs) == 1:
            return df
        else:
            return dfs


    def remove(self, kind, keys):
        '''
        Remove loaded indicators or activities.

        Parameters
        ----------
        kind : str
            Can be "indicator" or "activity".
        keys : iterable
            Key(s) of the item(s) to be removed.
        '''

        kind_lower = kind.lower()
        num = 0
        if kind_lower in ('indicator', 'indicators'):
            for k in keys:
                self._indicators.remove(k)
                num += 1
            msg = 'indicators' if num > 1 else 'indicator'

        elif kind_lower in ('activity', 'activities'):
            for k in keys:
                self._activities.pop(k)
                num += 1
            msg = 'activities' if num > 1 else 'activity'

        else:
            raise ValueError('kind can only be "indicator" or "activity", ' \
                             f'not "{kind}".')

        print(f'Successfully removed {num} {msg} from {self.name}.')

    def export_indicators(self, indicators=(), aliases={}, descriptions={},
                          name_formatter=None, alias_formatter=None,
                          show=False, path=''):
        '''
        Show information about the loaded impact indicators in a :class`pandas.DataFrame`,
        the :class`pandas.DataFrame` will be exported to the path if provided.

        Parameters
        ----------
        indicators : iterable
            Keys of the indicators in the `indicators` property.
            Will be defaulted to all loaded indicators if not provided.
        aliases : dict
            Keys should be the keys of the indicators in the `indicators` property,
            values should be the aliases of the indicators.
        descriptions : dict
            Keys should be the keys of the indicators in the `indicators` property,
            values should be the descriptions of the indicators.
        show : bool
            Whether to print the generated :class:`pandas.DataFrame` in the console.
        path : str
            If provided, the :class:`pandas.DataFrame` will be saved to the given file path.

        Returns
        -------
        df: :class:`pandas.DataFrame`
            Characterization factors.

        Tip
        ---
        The exported file can be directly import into QSD packages to create
        :class:`ImpactIndicator` items.


        '''
        if not self.indicators:
            raise ValueError('No loaded indicators.')
        if not set(indicators).issubset(self.indicators):
            raise ValueError('Provided indicator(s) not all loaded.')

        inds = indicators if indicators else self.indicators
        name_formatter = name_formatter or format_name
        if alias_formatter is None:
            alias_formatter = lambda ind: aliases.pop(ind, '')

        df = pd.DataFrame({
            'indicator': [name_formatter(ind[2]) for ind in inds],
            'alias': [alias_formatter(ind) for ind in inds],
            'unit': [bw2.methods.get(i)['unit'] for i in inds],
            'method': [ind[0] for ind in inds],
            'category': [ind[1] for ind in inds],
            'description': [descriptions[ind] if ind in descriptions.keys() else '' for ind in inds],
            'full_name': inds
            })

        export_df(df, path, show)

        return df


    def get_CFs(self, indicators=(), activities=(), show=False, path=''):
        '''
        Get impact characterization factors.

        Parameters
        ----------
        indicators : iterable
            Keys of the indicators in the `indicators` property.
            Will be defaulted to all loaded indicators if not provided.
        activities : iterable
            Keys of the activities in the `activities` property.
            Will be defaulted to all loaded indicators if not provided.
        show : bool
            Whether to print all characterization factors in the console.
        path : str
            If provided, the :class:`pandas.DataFrame` will be saved to the given file path.

        Returns
        -------
        df: :class:`pandas.DataFrame`
            Characterization factors.

        Tip
        ---
        [1] Use `show_activity` to see the functional unit of the activity.
        The quantity will be 1.

        [2] If you run into an "FileNotFoundError", most likely there are some
        indicators that do not acutally have corresponding impact assessment methods,
        try to load indicators one at a time to look for the culprit.

        '''

        if not self.indicators:
            raise ValueError('No loaded indicators.')
        elif not self.activities:
            raise ValueError('No loaded activities.')

        if not set(indicators).issubset(self.indicators):
            raise ValueError('Provided indicator(s) not all loaded.')

        inds = indicators if indicators else self.indicators
        ind_units = [bw2.methods.get(i)['unit'] for i in inds] + ['-', '-']
        acts = [self.activities[k] for k in activities] if activities \
            else [v for v in self.activities.values()]

        bw2.calculation_setups['multiLCA'] = {'inv': [{a:1} for a in acts],
                                              'ia': inds}

        # This is to prevent bw2 from printing in the console
        stdout = sys.stdout
        sys.stdout = open(os.devnull, 'w')
        mlca = bw2.MultiLCA('multiLCA')
        sys.stdout = stdout

        # pd_indices = [a['name'] for a in acts]
        pd_cols = pd.MultiIndex.from_tuples(inds, names=('method', 'category', 'indicator'))
        cf_df = pd.DataFrame(data=mlca.results, columns=pd_cols)
        unit_df = pd.DataFrame(data={k: v for k, v in zip(cf_df.columns, ind_units)},
                               index=[-1,], dtype='object')
        # cf_df = pd.DataFrame(data=mlca.results, index=pd_indices, columns=pd_cols)
        cf_df[('-', '-', 'activity name')] = [a['name'] for a in acts]
        cf_df[('-', '-', 'functional unit')] = [a['unit'] for a in acts]

        df = pd.concat((unit_df, cf_df))
        df.sort_index(axis=1, inplace=True)
        df.reset_index(drop=True, inplace=True)

        if show:
            print(df)

        export_df(df, path, show)
        self._CFs = df

        return df

    def get_CF(self, indicators=(), activities=(), show=False, path=''):
        '''Has been deprecated, use :func:`get_CF` instead.'''
        warn('`get_CF` has been deprecated, please use `get_CFs` instead.')
        return self.get_CFs(indicators, activities, show, path)



    def copy(self, name, omit=()):
        '''
        Return a copy of the :class:`CFgetter`.

        .. note:

            `CFs` is not copied.


        Parameters
        ----------
        name : str
            Name of the copy.
        omit: str or Iterable
            Property(ies) not to be copied.

        '''
        new = self.__class__.__new__(self.__class__)
        new.name = name
        new._CFs = None
        if isinstance(omit, str):
            omit = (omit,)
        omit = (*(f'_{o}' for o in omit), 'name', '_CFs')

        for s in self.__slots__:
            if not s in omit:
                if s == '_database':
                    new._database = self._database
                else:
                    setattr(new, s, getattr(self, s).copy())

        return new

    __copy__ = copy

    @property
    def available_databases(self):
        '''All databases that have been loaded into `Brightway2`.'''
        return bw2.databases

    @property
    def database(self):
        '''Loaded database.'''
        return self._database
    @database.setter
    def database(self, i):
        raise AttributeError('Use `load_database` to load/switch database.')

    @property
    def indicators(self):
        '''[list] Loaded impact indicators.'''
        return list(self._indicators)
    @indicators.setter
    def indicators(self, i):
        raise AttributeError('Use `load_indicators`/`remove` to add/remove indicators.')

    @property
    def activities(self):
        '''[dict] Loaded activities.'''
        return self._activities
    @activities.setter
    def activities(self, i):
        raise AttributeError('Use `load_activities`/`remove` to add/remove activities.')

    @property
    def CFs(self):
        '''Results from :func:`get_CFs`.'''
        if not self._CFs is None:
            return self._CFs
        else:
            return self.get_CFs()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
BW2QSD: Bridging Brightway2 and QSD packages for LCA

This module is developed by:

    Yalin Li <mailto.yalin.li@gmail.com>

This module is under the University of Illinois/NCSA Open Source License.
Please refer to https://github.com/QSD-Group/BW2QSD/blob/main/LICENSE.txt
for license details.
'''

from setuptools import setup

setup(
    name='bw2qsd',
    packages=['bw2qsd'],
    version='0.1.4',
    license='UIUC',
    author='Yalin Li',
    author_email='mailto.yalin.li@gmail.com',
    description='Bridging Brightway2 and QSD packages for LCA',
    long_description=open('README.rst', encoding='utf-8').read(),
    url="https://github.com/QSD-Group/BW2QSD",
    install_requires=[
        'brightway2==2.4.2',
        'beautifulsoup4',
        'py7zr',
    ],
    package_data=
        {'bw2qsd': [
            'eidl/*',
        ]},
    platforms=['Windows', 'Mac', 'Linux'],
    classifiers=[
        'License :: OSI Approved :: University of Illinois/NCSA Open Source License',
        'Environment :: Console',
        'Topic :: Education',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Chemistry',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Manufacturing',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    keywords=[
        'quantitative sustainable design',
        'Brightway2',
        'life cycle assessment',
        'database importer'
    ],
)
